@@ -1,11 +1,32 @@
 from __future__ import annotations
 
-from sqlglot import exp, generator, parser
+from sqlglot import exp, generator, parser, tokens
 from sqlglot.dialects.dialect import Dialect
 from sqlglot.tokens import TokenType
 
 
 class Teradata(Dialect):
+    class Tokenizer(tokens.Tokenizer):
+        # https://docs.teradata.com/r/Teradata-Database-SQL-Functions-Operators-Expressions-and-Predicates/March-2017/Comparison-Operators-and-Functions/Comparison-Operators/ANSI-Compliance
+        KEYWORDS = {
+            **tokens.Tokenizer.KEYWORDS,
+            "BYTEINT": TokenType.SMALLINT,
+            "SEL": TokenType.SELECT,
+            "INS": TokenType.INSERT,
+            "MOD": TokenType.MOD,
+            "LT": TokenType.LT,
+            "LE": TokenType.LTE,
+            "GT": TokenType.GT,
+            "GE": TokenType.GTE,
+            "^=": TokenType.NEQ,
+            "NE": TokenType.NEQ,
+            "NOT=": TokenType.NEQ,
+        }
+
+        # teradata does not support % for modulus
+        SINGLE_TOKENS = {**tokens.Tokenizer.SINGLE_TOKENS}
+        SINGLE_TOKENS.pop("%")
+
     class Parser(parser.Parser):
         CHARSET_TRANSLATORS = {
             "GRAPHIC_TO_KANJISJIS",
@@ -42,6 +63,14 @@ class Teradata(Dialect):
             "UNICODE_TO_UNICODE_NFKD",
         }
 
+        FUNC_TOKENS = {*parser.Parser.FUNC_TOKENS}
+        FUNC_TOKENS.remove(TokenType.REPLACE)
+
+        STATEMENT_PARSERS = {
+            **parser.Parser.STATEMENT_PARSERS,  # type: ignore
+            TokenType.REPLACE: lambda self: self._parse_create(),
+        }
+
         FUNCTION_PARSERS = {
             **parser.Parser.FUNCTION_PARSERS,  # type: ignore
             "TRANSLATE": lambda self: self._parse_translate(self.STRICT_CAST),
@@ -76,6 +105,14 @@ class Teradata(Dialect):
             )
 
     class Generator(generator.Generator):
+        PROPERTIES_LOCATION = {
+            **generator.Generator.PROPERTIES_LOCATION,  # type: ignore
+            exp.PartitionedByProperty: exp.Properties.Location.POST_INDEX,
+        }
+
+        def partitionedbyproperty_sql(self, expression: exp.PartitionedByProperty) -> str:
+            return f"PARTITION BY {self.sql(expression, 'this')}"
+
         # FROM before SET in Teradata UPDATE syntax
         # https://docs.teradata.com/r/Enterprise_IntelliFlex_VMware/Teradata-VantageTM-SQL-Data-Manipulation-Language-17.20/Statement-Syntax/UPDATE/UPDATE-Syntax-Basic-Form-FROM-Clause
         def update_sql(self, expression: exp.Update) -> str:
@@ -85,3 +122,6 @@ class Teradata(Dialect):
             where_sql = self.sql(expression, "where")
             sql = f"UPDATE {this}{from_sql} SET {set_sql}{where_sql}"
             return self.prepend_ctes(expression, sql)
+
+        def mod_sql(self, expression: exp.Mod) -> str:
+            return self.binary(expression, "MOD")
