@@ -1,3 +1,9 @@
+"""
+.. include:: ../posts/sql_diff.md
+
+----
+"""
+
 from __future__ import annotations
 
 import typing as t
@@ -9,10 +15,6 @@ from sqlglot import Dialect
 from sqlglot import expressions as exp
 from sqlglot.helper import ensure_collection
 
-if t.TYPE_CHECKING:
-    T = t.TypeVar("T")
-    Edit = t.Union[Insert, Remove, Move, Update, Keep]
-
 
 @dataclass(frozen=True)
 class Insert:
@@ -51,6 +53,11 @@ class Keep:
     target: exp.Expression
 
 
+if t.TYPE_CHECKING:
+    T = t.TypeVar("T")
+    Edit = t.Union[Insert, Remove, Move, Update, Keep]
+
+
 def diff(source: exp.Expression, target: exp.Expression) -> t.List[Edit]:
     """
     Returns the list of changes between the source and the target expressions.
@@ -71,12 +78,13 @@ def diff(source: exp.Expression, target: exp.Expression) -> t.List[Edit]:
         ]
 
     Args:
-        source (sqlglot.Expression): the source expression.
-        target (sqlglot.Expression): the target expression against which the diff should be calculated.
+        source: the source expression.
+        target: the target expression against which the diff should be calculated.
 
     Returns:
-        the list of Insert, Remove, Move, Update and Keep objects for each node in the source and the target expression trees.
-        This list represents a sequence of steps needed to transform the source expression tree into the target one.
+        the list of Insert, Remove, Move, Update and Keep objects for each node in the source and the
+        target expression trees. This list represents a sequence of steps needed to transform the source
+        expression tree into the target one.
     """
     return ChangeDistiller().diff(source.copy(), target.copy())
 
@@ -254,7 +262,7 @@ class ChangeDistiller:
         return bigram_histo
 
 
-def _get_leaves(expression: exp.Expression) -> t.Generator[exp.Expression, None, None]:
+def _get_leaves(expression: exp.Expression) -> t.Iterator[exp.Expression]:
     has_child_exprs = False
 
     for a in expression.args.values():
