@@ -1,10 +1,11 @@
 import itertools
 
 from sqlglot import alias, exp
-from sqlglot.optimizer.scope import traverse_scope
+from sqlglot.helper import csv_reader
+from sqlglot.optimizer.scope import Scope, traverse_scope
 
 
-def qualify_tables(expression, db=None, catalog=None):
+def qualify_tables(expression, db=None, catalog=None, schema=None):
     """
     Rewrite sqlglot AST to have fully qualified tables.
 
@@ -18,11 +19,14 @@ def qualify_tables(expression, db=None, catalog=None):
         expression (sqlglot.Expression): expression to qualify
         db (str): Database name
         catalog (str): Catalog name
+        schema: A schema to populate
     Returns:
         sqlglot.Expression: qualified expression
     """
     sequence = itertools.count()
 
+    next_name = lambda: f"_q_{next(sequence)}"
+
     for scope in traverse_scope(expression):
         for derived_table in scope.ctes + scope.derived_tables:
             if not derived_table.args.get("alias"):
@@ -41,12 +45,27 @@ def qualify_tables(expression, db=None, catalog=None):
                         source.set("catalog", exp.to_identifier(catalog))
 
                 if not source.alias:
-                    source.replace(
+                    source = source.replace(
                         alias(
                             source.copy(),
-                            source.this if identifier else f"_q_{next(sequence)}",
+                            source.this if identifier else next_name(),
                             table=True,
                         )
                     )
 
+                if schema and isinstance(source.this, exp.ReadCSV):
+                    with csv_reader(source.this) as reader:
+                        header = next(reader)
+                        columns = next(reader)
+                        schema.add_table(
+                            source, {k: type(v).__name__ for k, v in zip(header, columns)}
+                        )
+            elif isinstance(source, Scope) and source.is_udtf:
+                udtf = source.expression
+                table_alias = udtf.args.get("alias") or exp.TableAlias(this=next_name())
+                udtf.set("alias", table_alias)
+
+                if not table_alias.name:
+                    table_alias.set("this", next_name())
+
     return expression
